@@ -1,7 +1,7 @@
 /*
  * Seahorse
  *
- * Copyright (C) 2004 Nate Nielsen
+ * Copyright (C) 2004 Stefan Walter
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,6 @@
 #include "seahorse-context.h"
 #include "seahorse-gconf.h"
 #include "seahorse-widget.h"
-#include "seahorse-secure-memory.h"
 #include "seahorse-util.h"
  
 /* 
@@ -89,9 +88,6 @@ static void
 clear_clicked (GtkButton *button, SeahorseWidget *swidget)
 {
     seahorse_agent_cache_clearall ();
-#ifdef WITH_SSH
-    seahorse_agent_ssh_clearall ();
-#endif
     window_destroy ();
 }
 
@@ -159,11 +155,6 @@ window_update_keys ()
     
     /* The keys from the PGP cache */
     add_keys_to_store (store, seahorse_agent_cache_get_keys ());
-    
-#ifdef WITH_SSH
-    /* And  the ones from the SSH agent */
-    add_keys_to_store (store, seahorse_agent_ssh_cached_keys ());
-#endif
 }
 
 /* Display the status window */
@@ -178,7 +169,7 @@ window_show ()
         return;
     }
 
-    g_window = seahorse_widget_new ("agent-cache");
+    g_window = seahorse_widget_new ("agent-cache", NULL);
     w = glade_xml_get_widget (g_window->xml, g_window->name);
 
     g_signal_connect (G_OBJECT (w), "delete_event", G_CALLBACK (delete_event), NULL);
@@ -188,14 +179,6 @@ window_show ()
     glade_xml_signal_connect_data (g_window->xml, "clear_clicked",
                                    G_CALLBACK (clear_clicked), g_window);
 
-    /* The secure memory warning */
-    if (!seahorse_secure_memory_have ()) {
-        GdkColor color = { 0, 0xffff, 0, 0 };
-        w = glade_xml_get_widget (g_window->xml, "insecure_label");
-        gtk_widget_modify_fg (w, GTK_STATE_NORMAL, &color);
-        gtk_widget_show (w);
-    }
-
     window_update_keys ();
 }
 
@@ -217,12 +200,25 @@ on_show_window_activate (GtkWidget *item, gpointer data)
     window_show ();
 }
 
+static void 
+on_settings_childsetup (gpointer unused)
+{
+    seahorse_agent_childsetup ();
+}
+
 static void
 on_settings_activate (GtkWidget *item, gpointer data)
 {
     GError *err = NULL;
-    g_spawn_command_line_async ("seahorse-preferences --cache", &err);
-    
+    gchar* args[3];
+
+    args[0] = "seahorse-preferences";
+    args[1] = "--cache";
+    args[2] = NULL;
+
+    g_spawn_async (NULL, args, NULL, G_SPAWN_SEARCH_PATH, 
+                   on_settings_childsetup, NULL, NULL, &err);
+
     if (err != NULL) {
         g_warning ("couldn't execute seahorse-preferences: %s", err->message);
         g_error_free (err);
@@ -274,8 +270,6 @@ docklet_destroy ()
 static void
 docklet_create ()
 {
-    GtkWidget *box;
-
     if (g_docklet) {
         /* 
          * If this is being called when a tray icon exists, it's 
@@ -340,11 +334,7 @@ seahorse_agent_status_update ()
 
     if (have) {
         /* Only show when allowed to display, and have cached keys */
-        have = (seahorse_agent_cache_count () > 0)
-#ifdef WITH_SSH
-                || (seahorse_agent_ssh_count_keys () > 0)
-#endif 
-        ;
+        have = (seahorse_agent_cache_count () > 0);
     }
     
     if (have && !g_docklet)
