@@ -2,7 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2003 Jacob Perkins
- * Copyright (C) 2004 - 2006 Nate Nielsen
+ * Copyright (C) 2004 - 2006 Stefan Walter
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -178,6 +178,8 @@ seahorse_passphrase_prompt_show (const gchar *title, const gchar *description,
         prompt = _("Password:"); 
     
     w = gtk_dialog_new_with_buttons (title, NULL, GTK_DIALOG_NO_SEPARATOR, NULL);
+    gtk_window_set_icon_name (GTK_WINDOW (w), GTK_STOCK_DIALOG_AUTHENTICATION);
+
     dialog = GTK_DIALOG (w);
 
     g_signal_connect (G_OBJECT (dialog), "size-request",
@@ -514,24 +516,13 @@ gpg_detect_agent ()
  * SSH AGENT 
  */
 
-const guchar REQ_MESSAGE[] = { 0x00, 0x00, 0x00, 0x01, SEAHORSE_SSH_PING_MSG };
-const guint RESP_INDEX = 4;
-const guint RESP_VALUE = SEAHORSE_SSH_PING_MSG;
-
 static SeahorseAgentType
 ssh_detect_agent ()
 {
     SeahorseAgentType ret;
     const gchar *socketpath;
     struct sockaddr_un sunaddr;
-    guchar buf[16];
-    GIOChannel *io;
     int agentfd;
-    gsize bytes_read = 0;
-    GError *err = NULL;
-    
-    /* Guarantee we have enough space */
-    g_assert (sizeof (buf) > RESP_INDEX);
     
     socketpath = g_getenv ("SSH_AUTH_SOCK");
     if (!socketpath || !socketpath[0])
@@ -553,30 +544,10 @@ ssh_detect_agent ()
         close (agentfd);
         return SEAHORSE_AGENT_UNKNOWN;
     }
-    
-    io = g_io_channel_unix_new (agentfd);
-    g_io_channel_set_close_on_unref (io, TRUE);
-    g_io_channel_set_encoding (io, NULL, NULL);
-    g_io_channel_set_buffered (io, FALSE);
 
-    /* Send and receive our message */
-    if (g_io_channel_write_chars (io, (const gchar*)REQ_MESSAGE, sizeof (REQ_MESSAGE), NULL, 
-                                  &err) == G_IO_STATUS_NORMAL) {
-        g_io_channel_read_chars (io, (gchar*)buf, sizeof (buf), &bytes_read, &err);
-    }
-    
-    if (!err) {
-        if (bytes_read > RESP_INDEX && buf[RESP_INDEX] == RESP_VALUE)
-            ret = SEAHORSE_AGENT_SEAHORSE;
-        else
-            ret = SEAHORSE_AGENT_OTHER;
-    } else {
-        g_warning ("couldn't check for SSH agent: %s", err ? err->message : "");
-        ret = SEAHORSE_AGENT_UNKNOWN;
-    }
-    
-    g_io_channel_unref (io);
-    return ret;
+    shutdown (agentfd, SHUT_RDWR);
+    close (agentfd);
+    return SEAHORSE_AGENT_OTHER; 
 }
 
 /* -------------------------------------------------------------------------- */
