@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: seahorse-extension.c 1730 2007-04-16 16:25:52Z nnielsen $
+ *  $Id: seahorse-extension.c 2063 2008-01-31 03:18:40Z sadam $
  */
 
 #include "config.h"
@@ -567,6 +567,14 @@ build_ui (WindowData *data)
 	gtk_ui_manager_ensure_update (manager);
 }
 
+
+#if EPHY_NOTAB == 1
+static void
+impl_attach_tab (EphyExtension *extension,
+		 EphyWindow *window,
+		 EphyEmbed *embed)
+{
+#else
 static void
 impl_attach_tab (EphyExtension *extension,
 		 EphyWindow *window,
@@ -575,12 +583,20 @@ impl_attach_tab (EphyExtension *extension,
 	EphyEmbed *embed;
 
 	embed = ephy_tab_get_embed (tab);
+#endif
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
 	g_signal_connect (embed, "ge_context_menu",
 			  G_CALLBACK (context_menu_cb), window);
 }
 
+#if EPHY_NOTAB == 1
+static void
+impl_detach_tab (EphyExtension *extension,
+		 EphyWindow *window,
+		 EphyEmbed *embed)
+{
+#else
 static void
 impl_detach_tab (EphyExtension *extension,
 		 EphyWindow *window,
@@ -589,6 +605,7 @@ impl_detach_tab (EphyExtension *extension,
 	EphyEmbed *embed;
 
 	embed = ephy_tab_get_embed (tab);
+#endif
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
 	g_signal_handlers_disconnect_by_func
